'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ParameterInjector = function () {
    /**
     * Constructor
     *
     * Sets random delimiter
     *
     * @private
     *
     * @return {null} Nothin'
     */
    function ParameterInjector() {
        _classCallCheck(this, ParameterInjector);

        this.delimiter = (Math.random() * 1e64).toString(36);
    }

    /**
     * Static call to inject params into an object
     *
     * @private
     *
     * @param {string|object} target Plain object or string
     * @param {array|object} params Array or plain object with params
     *
     * @return {string|object} Plain object or string with injected params
     */


    _createClass(ParameterInjector, [{
        key: 'injectIntoObject',


        /**
         * Injects params into an object
         *
         * @private
         *
         * @param {object} obj Plain object
         * @param {array|object} params Array or plain object with params
         *
         * @return {object} Plain object with injected params
         */
        value: function injectIntoObject(obj, params) {
            var preparedString = this.injectionPreparation(obj);
            var newString = this.injectParams(preparedString, params);
            var newObject = this.injectionPreparation(newString);

            return newObject;
        }

        /**
         * Makes an object rea
         *
         * @private
         *
         * @param {string|object} obj Plain object or obj JSON string
         *
         * @return {string|object} Obj JSON string or object
         */

    }, {
        key: 'injectionPreparation',
        value: function injectionPreparation(obj) {
            if (_lodash2.default.isString(obj) && obj.indexOf(this.delimiter) !== -1) {
                var objArray = obj.split(this.delimiter);

                return JSON.parse(objArray[1]);
            }

            return this.delimiter + JSON.stringify(obj) + this.delimiter;
        }

        /**
         * Injects parameters into a string
         *
         * @private
         *
         * @param {string} str The target string
         * @param {string|array|object} params The params to be injected into the target string
         *
         * @return {string} Target string with injected parameters
         */

    }, {
        key: 'injectParams',
        value: function injectParams(str, params) {
            if (_lodash2.default.isPlainObject(params)) {
                return this.injectParamsObject(str, params);
            }

            if (!_lodash2.default.isArray(params)) {
                params = [params];
            }

            return this.injectParamsArray(str, params);
        }

        /**
         * Injects array of parameters into the target string
         *
         * @private
         * @source https://github.com/sequelize/sequelize/blob/master/lib/sql-string.js -> SqlString.format
         *
         * @param {string} str The target string
         * @param {array} params The params to be injected into the target string
         *
         * @return {string} Target string with injected parameters
         */

    }, {
        key: 'injectParamsArray',
        value: function injectParamsArray(str, params) {
            return str.replace(/\?/g, function (match) {
                if (!params.length) {
                    return match;
                }

                return params.shift();
            });
        }

        /**
         * Injects object of parameters into the target string
         *
         * @private
         * @source https://github.com/sequelize/sequelize/blob/master/lib/sql-string.js -> SqlString.formatNamedParameters
         *
         * @param {string} str The target string
         * @param {object} params The params to be injected into the target string
         *
         * @return {string} Target string with injected parameters
         */

    }, {
        key: 'injectParamsObject',
        value: function injectParamsObject(str, params) {
            return str.replace(/\:+(?!\d)(\w+)/g, function (value, key) {
                if (params[key] !== undefined) {
                    return params[key];
                }

                throw new Error('[1438415385] Named parameter "' + value + '" has no value in the given object.');
            });
        }
    }], [{
        key: 'inject',
        value: function inject(target, params) {
            var instance = new ParameterInjector();

            if (_lodash2.default.isString(target)) {
                return instance.injectParams(target, params);
            }

            if (_lodash2.default.isPlainObject(target) || _lodash2.default.isArray(target)) {
                return instance.injectIntoObject(target, _lodash2.default.clone(params));
            }

            return target;
        }
    }]);

    return ParameterInjector;
}();

exports.default = ParameterInjector;

